use warnings;
use strict;
use Test::More tests => 2;
use Test::Exception;
use Test::MockModule;

use POWER::Event;
use POWER::Event::IO;
use POWER::Utils::IO;


POWER::Event::IO->new({
    host        => 'www.google.com',
    port        => 443,
    cb          => \&client,
    wait_for    => $EOF,
    in_buf_limit=> 102400,
    out_buf     => "GET / HTTP/1.0\nHost: www.google.com\n\n",
    plugin      => [
        ssl         => POWER::Event::IO::SSL::CLIENT->new({
            trusted_CA  => '/etc/ssl/certs/ca-certificates.crt',
            cb_validate => \&validate,
        }),
    ],
});

my @Wait = (
    ['validate',    'www.google.com'    ],
    ['client',      $EOF                ],
);

eval { loop() };
die "out of loop: $@";

sub validate {
    my ($certs, $ssl, $io) = ($_[0], @{ $_[1] });
    is_deeply(shift @Wait, ['validate', $certs->[0]{subject}{commonName}], 'validate');
    return 0;
}

sub client {
    my ($io) = @_;
    is_deeply(shift @Wait, ['client', $io->{events}], 'client');
    die "server unexpected event: $io->{err}" if $io->{events} != $EOF;
    exit;
}

