use warnings;
use strict;
BEGIN { use lib '..'; require 't/.lib.t' }

my $cipher  = "This Is A Secret Key!";
my $banner  = "Server ready";
my $msg     = "Test message!\0\n" x 10_000;
my $repeat  = 2;

@CheckPoint = (
    [ 'listener',   $ACCEPTED       ], 'accept incoming connection',
    [ 'client',     $IN             ], 'client: got server banner',
    [ 'client',     $banner         ], 'client: banner is correct',
((
    [ 'server',     $IN             ], 'server: got in',
    [ 'server',     $msg            ], 'server: requst is correct',
    [ 'client',     $IN             ], 'client: got in',
    [ 'client',     "echo: $msg"    ], 'client: reply is correct',
) x ($repeat-1)),

    [ 'client',     $SENT           ], 'client: request sent',
    [ 'server',     $IN             ], 'server: got last in',
    [ 'server',     $msg            ], 'server: requst is correct',
    [ 'server',     $EOF            ], 'server: got eof',
    [ 'server',     $SENT           ], 'server: reply sent',
    [ 'client',     $EOF            ], 'client: got eof',
    [ 'client',     "echo: $msg"    ], 'client: reply is correct',
);
plan tests => @CheckPoint/2;

POWER::Event::IO::LISTEN->new({
    host        => '127.0.0.1',
    port        => 4444,
    cb          => \&listener,
    wait_for    => $ACCEPTED,
});

POWER::Event::IO->new({
    host        => '127.0.0.1',
    port        => 4444,
    cb          => \&client,
    wait_for    => $IN,
    in_buf_limit=> 1024000,
    plugin      => [
        rc4         => POWER::Event::IO::CRYPT::RC4->new({
            cipher  => $cipher,
        }),
    ],
    Repeat      => $repeat,
});

eval { loop() };
die "out of loop: $@";

sub listener {
    my ($io) = @_;
    checkpoint($io->{events});
    POWER::Event::IO->new({
        fh          => delete $io->{in_buf},
        cb          => \&server,
        wait_for    => $IN|$EOF,
        in_buf_limit=> 1024000,
        out_buf     => $banner,
        plugin      => [
            rc4         => POWER::Event::IO::CRYPT::RC4->new({
                cipher  => $cipher,
            }),
        ],
        Repeat      => $repeat,
    });
}

sub server {
    my ($io) = @_;
#  &diag_event;
    return if $io->{events} == $IN && length $io->{in_buf} < length $msg;
    checkpoint($io->{events});
    if ($io->{events} & $ERR) {
        die $io->{err};
    }
    if ($io->{events} & $IN) {
        checkpoint($io->{in_buf});
        $io->wait_for($EOF|$SENT) if ! --$io->{Repeat};
        $io->write("echo: $io->{in_buf}");
        $io->{in_buf} = q{};
    }
    if ($io->{events} & $SENT) {
        shutdown $io->{fh}, 1;
    }
}

sub client {
    my ($io) = @_;
#  &diag_event;
    return
        if $io->{events} == $IN
        && length $io->{in_buf} != length $banner
        && length $io->{in_buf} < length $msg
        ;
    checkpoint($io->{events});
    if ($io->{events} & $IN) {
        checkpoint($io->{in_buf});
        $io->{in_buf} = q{};
        $io->wait_for($SENT|$EOF) if ! --$io->{Repeat};
        $io->write($msg);
    }
    if ($io->{events} & $SENT) {
        shutdown $io->{fh}, 1;
    }
    if ($io->{events} & $EOF) {
        checkpoint($io->{in_buf});
        exit;
    }
}

