use warnings;
use strict;
BEGIN { use lib '..'; require 't/.lib.t' }

if ($INC{'Devel/Cover.pm'}) {
$POWER::Event::IO::SSL::TO_SSL_HANDSHAKE = int 3   / $POWER::Event::IO::TICK_RATE + 0.5;
} else {
$POWER::Event::IO::SSL::TO_SSL_HANDSHAKE = int 0.3 / $POWER::Event::IO::TICK_RATE + 0.5;
}

my $banner  = "Server ready";
my $msg     = "Test message!\0\n";

@CheckPoint = (
    [ 'listener',       $ACCEPTED       ], 'accept incoming connection',
    [ 'ssl_client',     $ERR            ], 'client: handshake timed out',
    [ 'ssl_listener',   $ACCEPTED       ], 'accept incoming connection',
    [ 'ssl_server',     $ERR            ], 'server: handshake timed out',
);
plan tests => @CheckPoint/2;

POWER::Event::IO::LISTEN->new({
    host        => '127.0.0.1',
    port        => 4400,
    cb          => \&listener,
    wait_for    => $ACCEPTED,
});
POWER::Event::IO->new({
    host        => '127.0.0.1',
    port        => 4400,
    cb          => \&ssl_client,
    wait_for    => $IN,
    plugin      => [
        ssl         => POWER::Event::IO::SSL::CLIENT->new({
            in_buf_limit=> 1024,
            trusted_CA  => 't/cert/testca.crt',
        }),
    ],
});

eval { loop() };
die "out of loop: $@" if $@ !~ /timed out/;

POWER::Event::IO::LISTEN->new({
    host        => '127.0.0.1',
    port        => 4433,
    cb          => \&ssl_listener,
    wait_for    => $ACCEPTED,
});
POWER::Event::IO->new({
    host        => '127.0.0.1',
    port        => 4433,
    cb          => \&client,
    wait_for    => $IN,
    in_buf_limit=> 1024,
});

eval { loop() };
die "out of loop: $@" if $@ !~ /timed out/;


sub listener {
    my ($io) = @_;
    checkpoint($io->{events});
}

sub ssl_listener {
    my ($io) = @_;
    checkpoint($io->{events});
    POWER::Event::IO->new({
        fh          => delete $io->{in_buf},
        cb          => \&ssl_server,
        wait_for    => $EOF,
        in_buf_limit=> 1024,
        out_buf     => $banner,
        plugin      => [
            ssl         => POWER::Event::IO::SSL::SERVER->new({
                ssl_crt     => 't/cert/testsrv.crt',
                ssl_key     => 't/cert/testsrv.key',
            }),
        ],
    });
}

sub ssl_server {
    my ($io) = @_;
#  &diag_event;
    checkpoint($io->{events});
    if ($io->{events} & $ERR) {
        die $io->{err};
    }
}

sub client {
    my ($io) = @_;
#  &diag_event;
    checkpoint($io->{events});
    if ($io->{events} & $ERR) {
        die $io->{err};
    }
}

sub ssl_client {
    my ($io) = @_;
#  &diag_event;
    checkpoint($io->{events});
    if ($io->{events} & $ERR) {
        die $io->{err};
    }
}

