use warnings;
use strict;
use Test::More tests => 19;
use Test::Exception;
use Test::MockModule;

use POWER::Event;
use POWER::Event::IO;


my @Send = (
    [$IN,                       "test\n"],
    [$IN|$EOF|$ERR,             "test\n"],
    [$IN|$EOF|$ERR|$OUT,        "test\n"],
    [$IN|$EOF|$ERR|$SENT,       "test\n"],
    [$IN|$EOF|$ERR|$OUT|$SENT,  "test\n"],
    [$IN|$EOF|$ERR,             "test\n"],
    [$IN|$EOF|$ERR|$OUT|$SENT,  ("x" x 102400)."\n"],
    [$IN|$EOF|$ERR|$OUT|$SENT,  ("x" x 102400)."\n"],
    [$IN|$EOF|$ERR|$OUT|$SENT,  ("x" x 204800)."\n"],
    [$IN|$EOF|$ERR|$OUT,        ("x" x 204800)."\n"],
    [$IN|$EOF|$ERR|$SENT,       ("x" x 204800)."\n"],
    [$EOF,                      "send 204800\n" . "send eof\n"],
);
my @Wait = (
#    [$IN,                       "test\n"],
    [$IN, "test passed\n"],
#    [$IN|$EOF|$ERR,             "test\n"],
    [$IN, "test passed\n"],
#    [$IN|$EOF|$ERR|$OUT,        "test\n"],
    $OUT,
    [$IN, "test passed\n"],
#    [$IN|$EOF|$ERR|$SENT,       "test\n"],
    [$SENT, q{}],
    [$IN, "test passed\n"],
#    [$IN|$EOF|$ERR|$OUT|$SENT,  "test\n"],
    [$OUT|$SENT, q{}],
    [$IN, "test passed\n"],
#    [$IN|$EOF|$ERR,             "test\n"],
    [$IN, "test passed\n"],
#    [$IN|$EOF|$ERR|$OUT|$SENT,  ("x" x 102400)."\n"],
    $OUT,
    [$OUT|$SENT, q{}],
    [$IN, "102400 bytes received\n"],
#    [$IN|$EOF|$ERR|$OUT|$SENT,  ("x" x 102400)."\n"],
    [$OUT|$SENT, q{}],
    [$IN, "102400 bytes received\n"],
#    [$IN|$EOF|$ERR|$OUT|$SENT,  ("x" x 204800)."\n"],
    $OUT,
    [$OUT|$SENT, q{}],
    [$IN, "204800 bytes received\n"],
#    [$IN|$EOF|$ERR|$OUT,        ("x" x 204800)."\n"],
    $OUT,
    [$IN, "204800 bytes received\n"],
#    [$IN|$EOF|$ERR|$SENT,       ("x" x 204800)."\n"],
    [$SENT, q{}],
    [$IN, "204800 bytes received\n"],
#    [$EOF,                      "send 204800\n" . "send eof\n"],
    [$ERR, "in_buf_limit reached"],
    [$EOF, "got 204800 bytes\n"],
);

POWER::Event::IO::LISTEN->new({
    host        => '127.0.0.1',
    port        => 1234,
    cb          => \&listener,
    wait_for    => $ACCEPTED,
});
POWER::Event::IO->new({
    host        => '127.0.0.1',
    port        => 1234,
    cb          => \&client,
    wait_for    => $Send[0][0],
    out_buf     => $Send[0][1],
    in_buf_limit=> 102400,
});
shift @Send;

eval { loop() };
die "out of loop: $@";

sub listener {
    my ($io) = @_;
#    is_deeply(shift @Wait, ['listener', $io->{events}], 'listener');
    die "listener unexpected event: $io->{err}" if $io->{events} != $ACCEPTED;
    POWER::Event::IO->new({
        fh          => delete($io->{in_buf}),
        cb          => \&server,
        wait_for    => $IN,
    });
}

sub server {
    my ($io) = @_;
#    is_deeply(shift @Wait, ['server', $io->{events}], 'server');
    shutdown($io->{fh}, 1), return if $io->{events} == $SENT;
    die "server unexpected event: $io->{err}" if $io->{events} != $IN;
    while ($io->{in_buf} =~ s/\A([^\n]+)\n//xms) {
        my $cmd = $1;
        if ($cmd eq 'test') {
            $io->write("test passed\n");
        } elsif ($cmd =~ /\Asend\s+(\d+)\z/xms) {
            $io->write(('x' x ($1-1))."\n");
        } elsif ($cmd eq 'send eof') {
            $io->wait_for($IN|$SENT);
        } else {
            $io->write(length($cmd)." bytes received\n");
        }
    }
}

sub client {
    my ($io) = @_;
    if ($io->{events} == $ERR) {
        if ($io->{err} eq 'in_buf_limit reached') {
            is_deeply([$io->{events}, $io->{err}], shift(@Wait));
            $io->{in_buf_limit} = 0;
        }
        else {
            die "got error: $io->{err}";
        }
    }
    else {
        $io->{in_buf} = "got ".length($io->{in_buf})." bytes\n"
            if length($io->{in_buf}) > 100;
        if ($Wait[0]==$OUT and ($io->{events}==$OUT and $io->{in_buf} eq q{})) {
            # amount of $OUT is depend on size of system buffers, skip them all
        }
        else {
            shift @Wait if $Wait[0]==$OUT;
            is_deeply([$io->{events}, $io->{in_buf}], shift(@Wait));
        }
        $io->{in_buf} = q{};
    }
    exit if $io->{events} & $EOF;
    return if !($io->{events} & $IN);
    return unless @Send;
    $io->wait_for($Send[0][0]);
    $io->write($Send[0][1]);
    shift @Send;
}

